/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import React, { FC, useCallback, useEffect, useMemo, useState } from 'react';
import { useLocation } from 'react-router-dom';
import { BaseVisType, VisGroups, VisTypeAlias } from '@kbn/visualizations-plugin/public';
import {
  EmbeddableFactory,
  EmbeddableFactoryDefinition,
  EmbeddableInput,
} from '@kbn/embeddable-plugin/public';
import { trackCanvasUiMetric, METRIC_TYPE } from '../../../lib/ui_metric';
import { useEmbeddablesService, useVisualizationsService } from '../../../services';
import { CANVAS_APP } from '../../../../common/lib';
import { encode } from '../../../../common/lib/embeddable_dataurl';
import { ElementSpec } from '../../../../types';
import { EditorMenu as Component } from './editor_menu.component';

interface Props {
  /**
   * Handler for adding a selected element to the workpad
   */
  addElement: (element: Partial<ElementSpec>) => void;
}

interface UnwrappedEmbeddableFactory {
  factory: EmbeddableFactory;
  isEditable: boolean;
}

export const EditorMenu: FC<Props> = ({ addElement }) => {
  const embeddablesService = useEmbeddablesService();
  const { pathname, search, hash } = useLocation();
  const stateTransferService = embeddablesService.getStateTransfer();
  const visualizationsService = useVisualizationsService();

  const embeddableFactories = useMemo(
    () => (embeddablesService ? Array.from(embeddablesService.getEmbeddableFactories()) : []),
    [embeddablesService]
  );

  const [unwrappedEmbeddableFactories, setUnwrappedEmbeddableFactories] = useState<
    UnwrappedEmbeddableFactory[]
  >([]);

  useEffect(() => {
    Promise.all(
      embeddableFactories.map<Promise<UnwrappedEmbeddableFactory>>(async (factory) => ({
        factory,
        isEditable: await factory.isEditable(),
      }))
    ).then((factories) => {
      setUnwrappedEmbeddableFactories(factories);
    });
  }, [embeddableFactories]);

  const createNewVisType = useCallback(
    (visType?: BaseVisType | VisTypeAlias) => () => {
      let path = '';
      let appId = '';

      if (visType) {
        if (trackCanvasUiMetric) {
          trackCanvasUiMetric(METRIC_TYPE.CLICK, `${visType.name}:create`);
        }

        if (!('alias' in visType)) {
          // this visualization is not an alias
          appId = 'visualize';
          path = `#/create?type=${encodeURIComponent(visType.name)}`;
        } else if (visType.alias && 'path' in visType.alias) {
          // this visualization **is** an alias, and it has an app to redirect to for creation
          appId = visType.alias.app;
          path = visType.alias.path;
        }
      } else {
        appId = 'visualize';
        path = '#/create?';
      }

      stateTransferService.navigateToEditor(appId, {
        path,
        state: {
          originatingApp: CANVAS_APP,
          originatingPath: `${pathname}${search}${hash}`,
        },
      });
    },
    [stateTransferService, pathname, search, hash]
  );

  const createNewEmbeddable = useCallback(
    (factory: EmbeddableFactoryDefinition) => async () => {
      if (trackCanvasUiMetric) {
        trackCanvasUiMetric(METRIC_TYPE.CLICK, factory.type);
      }
      let embeddableInput;
      if (factory.getExplicitInput) {
        embeddableInput = await factory.getExplicitInput();
      } else {
        const newEmbeddable = await factory.create({} as EmbeddableInput);
        embeddableInput = newEmbeddable?.getInput();
      }

      if (embeddableInput) {
        const config = encode(embeddableInput);
        const expression = `embeddable config="${config}"
  type="${factory.type}"
| render`;

        addElement({ expression });
      }
    },
    [addElement]
  );

  const getVisTypesByGroup = (group: VisGroups): BaseVisType[] =>
    visualizationsService
      .getByGroup(group)
      .sort(({ name: a }: BaseVisType | VisTypeAlias, { name: b }: BaseVisType | VisTypeAlias) => {
        if (a < b) {
          return -1;
        }
        if (a > b) {
          return 1;
        }
        return 0;
      })
      .filter(({ disableCreate }: BaseVisType) => !disableCreate);

  const visTypeAliases = visualizationsService
    .getAliases()
    .sort(({ promotion: a = false }: VisTypeAlias, { promotion: b = false }: VisTypeAlias) =>
      a === b ? 0 : a ? -1 : 1
    )
    .filter(({ disableCreate }: VisTypeAlias) => !disableCreate);

  const factories = unwrappedEmbeddableFactories
    .filter(
      ({ isEditable, factory: { type, canCreateNew, isContainerType } }) =>
        isEditable &&
        !isContainerType &&
        canCreateNew() &&
        !['visualization', 'ml', 'links'].some((factoryType) => {
          return type.includes(factoryType);
        })
    )
    .map(({ factory }) => factory);

  const promotedVisTypes = getVisTypesByGroup(VisGroups.PROMOTED);

  return (
    <Component
      createNewVisType={createNewVisType}
      createNewEmbeddable={createNewEmbeddable}
      promotedVisTypes={promotedVisTypes}
      factories={factories}
      visTypeAliases={visTypeAliases}
    />
  );
};
